#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : scan.py
# Author             : Podalirius (@podalirius_)
# Date created       : 29 Jul 2022

import base64
import datetime
import re
import time
from apachetomcatscanner.utils.network import is_port_open, is_http_accessible


import requests
# Disable warnings of insecure connection for invalid certificates
requests.packages.urllib3.disable_warnings()
# Allow use of deprecated and weak cipher methods
requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS += ':HIGH:!DH:!aNULL'
try:
    requests.packages.urllib3.contrib.pyopenssl.util.ssl_.DEFAULT_CIPHERS += ':HIGH:!DH:!aNULL'
except AttributeError:
    pass


def is_tomcat_manager_accessible(target, port, config, scheme="http"):
    path = "/manager/html"
    url = "%s://%s:%d%s" % (scheme, target, port, path)
    try:
        r = requests.get(
            url,
            timeout=config.request_timeout,
            proxies=config.request_proxies,
            verify=(not (config.request_no_check_certificate))
        )
        if r.status_code in [401]:
            return True
        else:
            return False
    except Exception as e:
        config.debug("Error in is_tomcat_manager_accessible('%s', %d, '%s'): %s " % (target, port, scheme, e))
        return False


def get_version_from_malformed_http_request(target, port, config, scheme="http"):
    url = "%s://%s:%d/{}" % (scheme, target, port)
    try:
        r = requests.get(
            url,
            timeout=config.request_timeout,
            proxies=config.request_proxies,
            verify=(not (config.request_no_check_certificate))
        )
    except Exception as e:
        config.debug("Error in get_version_from_malformed_http_request('%s', %d, '%s'): %s " % (target, port, scheme, e))
        return None
    if r.status_code in [400, 404, 500]:
        # Bug triggered
        matched = re.search(b"(<h3>)Apache Tomcat(/)?([^<]+)(</h3>)", r.content)
        if matched is not None:
            _, _, version, _ = matched.groups()
            version = version.decode('utf-8')
            return version


def try_default_credentials(target, port, config, scheme="http"):
    found_credentials = []
    url = "%s://%s:%d/manager/html" % (scheme, target, port)
    try:
        for credentials in config.credentials:
            auth_string = bytes(credentials["username"] + ':' + credentials["password"], 'utf-8')
            r = requests.post(
                url,
                headers={
                    "Authorization": "Basic " + base64.b64encode(auth_string).decode('utf-8')
                },
                timeout=config.request_timeout,
                proxies=config.request_proxies,
                verify=(not (config.request_no_check_certificate))
            )
            if r.status_code in [200, 403]:
                found_credentials.append((r.status_code, credentials))
        return found_credentials
    except Exception as e:
        config.debug("Error in get_version_from_malformed_http_request('%s', %d, '%s'): %s " % (target, port, scheme, e))
        return found_credentials


def scan_worker(target, port, reporter, config, monitor_data):
    try:
        result = {"target": target}

        if is_port_open(target, port):
            for scheme in config.get_request_available_schemes():
                if is_http_accessible(target, port, config, scheme):
                    result["version"] = get_version_from_malformed_http_request(target, port, config, scheme)
                    if result["version"] is not None:
                        config.debug("Found version %s" % result["version"])

                        result["manager_accessible"] = is_tomcat_manager_accessible(target, port, config, scheme)

                        credentials_found = []
                        if result["manager_accessible"]:
                            config.debug("Manager is accessible")
                            # Test for default credentials
                            credentials_found = try_default_credentials(target, port, config, scheme)

                        reporter.report_result(
                            target,
                            port,
                            result["version"],
                            result["manager_accessible"],
                            credentials_found
                        )

        monitor_data["lock"].acquire()
        monitor_data["actions_performed"] = monitor_data["actions_performed"] + 1
        # print("Updated for port %d" % port)
        monitor_data["lock"].release()

    except Exception as e:
        if config.debug_mode:
            print("[Error in %s] %s" % (__name__, e))


def monitor_thread(reporter, config, monitor_data):
    last_check, monitoring = 0, True
    while monitoring:
        new_check = monitor_data["actions_performed"]
        rate = (new_check - last_check)
        if not config.debug_mode:
            print("\r", end="")
        reporter.print_new_results()
        print("[%s] Status (%d/%d) %5.2f %% | Rate %d tests/s        " % (
                datetime.datetime.now().strftime("%Y/%m/%d %Hh%Mm%Ss"),
                new_check, monitor_data["total"], (new_check/monitor_data["total"])*100,
                rate
            ),
            end=("" if not config.debug_mode else "\n")
        )
        last_check = new_check
        time.sleep(1)
        if rate == 0 and monitor_data["actions_performed"] == monitor_data["total"]:
            monitoring = False

    if len(reporter._new_results) != 0:
        reporter.print_new_results()

    print()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : __main__.py
# Author             : Podalirius (@podalirius_)
# Date created       : 24 Jul 2022
import threading

import argparse
import os
import sys

from apachetomcatscanner.Reporter import Reporter
from apachetomcatscanner.Config import Config
from apachetomcatscanner.VulnerabilitiesDB import VulnerabilitiesDB
from apachetomcatscanner.utils.scan import scan_worker, monitor_thread
from sectools.windows.ldap import get_computers_from_domain, get_servers_from_domain, get_subnets
from sectools.network.domains import is_fqdn
from sectools.network.ip import is_ipv4_cidr, is_ipv4_addr, is_ipv6_addr, expand_cidr, expand_port_range
from concurrent.futures import ThreadPoolExecutor


VERSION = "3.0"

banner = """Apache Tomcat Scanner v%s - by @podalirius_\n""" % VERSION


def load_targets(options, config):
    targets = []

    # Loading targets from domain computers
    if options.auth_dc_ip is not None and options.auth_user is not None and (options.auth_password is not None or options.auth_hashes is not None) and options.servers_only is False:
        if options.debug:
            print("[debug] Loading targets from computers in the domain '%s'" % options.auth_domain)
        targets += get_computers_from_domain(
            auth_domain=options.auth_domain,
            auth_dc_ip=options.auth_dc_ip,
            auth_username=options.auth_user,
            auth_password=options.auth_password,
            auth_hashes=options.auth_hashes,
            use_ldaps=options.ldaps,
            __print=True
        )

    # Loading targets from domain servers
    if options.auth_dc_ip is not None and options.auth_user is not None and (options.auth_password is not None or options.auth_hashes is not None) and options.servers_only is True:
        if options.debug:
            print("[debug] Loading targets from servers in the domain '%s'" % options.auth_domain)
        targets += get_servers_from_domain(
            auth_domain=options.auth_domain,
            auth_dc_ip=options.auth_dc_ip,
            auth_username=options.auth_user,
            auth_password=options.auth_password,
            auth_hashes=options.auth_hashes,
            use_ldaps=options.ldaps,
            __print=True
        )

    # Loading targets from subnetworks of the domain
    if options.auth_dc_ip is not None and options.auth_user is not None and (options.auth_password is not None or options.auth_hashes is not None):
        if options.debug:
            print("[debug] Loading targets from servers in the domain '%s'" % options.auth_domain)
        targets += get_subnets(
            auth_domain=options.auth_domain,
            auth_dc_ip=options.auth_dc_ip,
            auth_username=options.auth_user,
            auth_password=options.auth_password,
            auth_hashes=options.auth_hashes,
            use_ldaps=options.ldaps,
            __print=True
        )

    # Loading targets line by line from a targets file
    if options.targets_file is not None:
        if os.path.exists(options.targets_file):
            if options.debug:
                print("[debug] Loading targets line by line from targets file '%s'" % options.targets_file)
            f = open(options.targets_file, "r")
            for line in f.readlines():
                targets.append(line.strip())
            f.close()
        else:
            print("[!] Could not open targets file '%s'" % options.targets_file)

    # Loading targets from a single --target option
    if len(options.target) != 0:
        if options.debug:
            print("[debug] Loading targets from --target options")
        for target in options.target:
            targets.append(target)

    # Sort uniq on targets list
    targets = sorted(list(set(targets)))

    final_targets = []
    # Parsing target to filter IP/DNS/CIDR
    for target in targets:
        if is_ipv4_cidr(target):
            final_targets += expand_cidr(target)
        elif is_ipv4_addr(target):
            final_targets.append(target)
        elif is_ipv6_addr(target):
            final_targets.append(target)
        elif is_fqdn(target):
            final_targets.append(target)
        else:
            if options.debug:
                print("[debug] Target '%s' was not added." % target)

    final_targets = sorted(list(set(final_targets)))
    return final_targets


def load_ports(options, config):
    ports = []
    if "," in options.target_ports:
        for port in options.target_ports.split(','):
            ports += expand_port_range(port.strip())
    else:
        ports = expand_port_range(options.target_ports.strip())
    ports = sorted(list(set(ports)))
    return ports


def parseArgs():
    print(banner)
    parser = argparse.ArgumentParser(description="A python script to scan for Apache Tomcat server vulnerabilities.")
    parser.add_argument("-v", "--verbose", default=False, action="store_true", help="Verbose mode. (default: False)")
    parser.add_argument("--debug", default=False, action="store_true", help="Debug mode, for huge verbosity. (default: False)")
    parser.add_argument("-C", "--list-cves", default=False, action="store_true", help="List CVE ids affecting each version found. (default: False)")
    parser.add_argument("-T", "--threads", default=250, type=int, help="Number of threads (default: 250)")
    parser.add_argument("-s", "--servers-only", default=False, action="store_true", help="If querying ActiveDirectory, only get servers and not all computer objects. (default: False)")

    parser.add_argument("--only-http", default=False, action="store_true", help="Scan only with HTTP scheme. (default: False, scanning with both HTTP and HTTPs)")
    parser.add_argument("--only-https", default=False, action="store_true", help="Scan only with HTTPs scheme. (default: False, scanning with both HTTP and HTTPs)")
    # parser.add_argument("--no-check-certificate", default=False, action="store_true", help="Do not check certificate. (default: False)")

    group_export = parser.add_argument_group("Export results")
    group_export.add_argument("--export-xlsx", dest="export_xlsx", type=str, default=None, required=False, help="Output XLSX file to store the results in.")
    group_export.add_argument("--export-json", dest="export_json", type=str, default=None, required=False, help="Output JSON file to store the results in.")
    group_export.add_argument("--export-sqlite", dest="export_sqlite", type=str, default=None, required=False, help="Output SQLITE3 file to store the results in.")

    group_configuration = parser.add_argument_group("Advanced configuration")
    group_configuration.add_argument("-PI", "--proxy-ip", default=None, type=str, help="Proxy IP.")
    group_configuration.add_argument("-PP", "--proxy-port", default=None, type=int, help="Proxy port")
    group_configuration.add_argument("-rt", "--request-timeout", default=1, type=int, help="Set the timeout of HTTP requests.")
    group_configuration.add_argument("--tomcat-username", default=None, help="Set the timeout of HTTP requests.")
    group_configuration.add_argument("--tomcat-usernames-file", default=None, help="Set the timeout of HTTP requests.")
    group_configuration.add_argument("--tomcat-password", default=None, help="Set the timeout of HTTP requests.")
    group_configuration.add_argument("--tomcat-passwords-file", default=None, help="Set the timeout of HTTP requests.")

    group_targets_source = parser.add_argument_group("Targets")
    group_targets_source.add_argument("-tf", "--targets-file", default=None, type=str, help="Path to file containing a line by line list of targets.")
    group_targets_source.add_argument("-tt", "--target", default=[], type=str, action='append', help="Target IP, FQDN or CIDR")
    group_targets_source.add_argument("-tp", "--target-ports", default="80,443,8080,8081,9080,9081,10080", type=str, help="Target ports to scan top search for Apache Tomcat servers.")
    group_targets_source.add_argument("-ad", "--auth-domain", default="", type=str, help="Windows domain to authenticate to.")
    group_targets_source.add_argument("-ai", "--auth-dc-ip", default=None, type=str, help="IP of the domain controller.")
    group_targets_source.add_argument("-au", "--auth-user", default=None, type=str, help="Username of the domain account.")
    group_targets_source.add_argument("-ap", "--auth-password", default=None, type=str, help="Password of the domain account.")
    group_targets_source.add_argument("-ah", "--auth-hashes", default=None, type=str, help="LM:NT hashes to pass the hash for this user.")
    group_targets_source.add_argument("--ldaps", default=False, action="store_true", help="Use LDAPS (default: False)")
    group_targets_source.add_argument("--subnets", default=False, action="store_true", help="Get all subnets from the domain and use them as targets (default: False)")

    args = parser.parse_args()

    if (args.targets_file is None) and (len(args.target) == 0) and (args.auth_user is None and (args.auth_password is None or args.auth_hashes is None)):
        parser.print_help()
        print("\n[!] No targets specified.")
        sys.exit(0)

    if (args.auth_password is not None) and (args.auth_hashes is not None):
        parser.print_help()
        print("\n[!] Options --auth-password/--auth-hashes are mutually exclusive.")
        sys.exit(0)

    if (args.auth_dc_ip is None) and (args.auth_user is not None and (args.auth_password is not None or args.auth_hashes is not None)):
        parser.print_help()
        print("\n[!] Option --auth-dc-ip is required when using --auth-user, --auth-password, --auth-hashes, --auth-domain")
        sys.exit(0)

    return args


def main():
    options = parseArgs()

    config = Config()
    config.set_debug_mode(options.debug)
    config.set_request_available_schemes(only_http=options.only_http, only_https=options.only_https)
    config.set_request_timeout(options.request_timeout)
    config.set_request_proxies(options.proxy_ip, options.proxy_port)
    # config.set_request_no_check_certificate(options.no_check_certificate)
    config.set_list_cves_mode(options.list_cves)

    config.load_credentials_from_options(options.tomcat_username, options.tomcat_password, options.tomcat_usernames_file, options.tomcat_passwords_file)

    vulns_db = VulnerabilitiesDB(config=config)
    reporter = Reporter(config=config, vulns_db=vulns_db)

    # Parsing targets and ports
    targets = load_targets(options, config)
    ports = load_ports(options, config)
    if len(targets) != 0:
        if len(ports) != 0:
            if options.proxy_ip is not None and options.proxy_port is not None:
                print("[+] Targeting %d ports on %d targets through proxy %s:%d" % (len(ports), len(targets), options.proxy_ip, options.proxy_port))
            else:
                print("[+] Targeting %d ports on %d targets" % (len(ports), len(targets)))

            # Exploring targets
            if len(targets) != 0 and options.threads != 0:
                print("[+] Searching for Apache Tomcats servers on specified targets ...")
                monitor_data = {"actions_performed": 0, "total": (len(targets)*len(ports)), "lock": threading.Lock()}
                with ThreadPoolExecutor(max_workers=min(options.threads, 1+monitor_data["total"])) as tp:
                    tp.submit(monitor_thread, reporter, config, monitor_data)
                    for target in targets:
                        for port in ports:
                            tp.submit(scan_worker, target, port, reporter, config, monitor_data)
                print("[+] All done!")

            if options.export_xlsx is not None:
                reporter.export_xlsx(options.export_xlsx)

            if options.export_json is not None:
                reporter.export_json(options.export_json)

            if options.export_sqlite is not None:
                reporter.export_sqlite(options.export_sqlite)
        else:
            print("[!] Cannot start scan: no ports loaded.")
    else:
        print("[!] Cannot start scan: no targets loaded.")


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : network.py
# Author             : Podalirius (@podalirius_)
# Date created       : 17 Jan 2023

import socket

import requests
# Disable warnings of insecure connection for invalid certificates
requests.packages.urllib3.disable_warnings()
# Allow use of deprecated and weak cipher methods
requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS += ':HIGH:!DH:!aNULL'
try:
    requests.packages.urllib3.contrib.pyopenssl.util.ssl_.DEFAULT_CIPHERS += ':HIGH:!DH:!aNULL'
except AttributeError:
    pass


def is_target_a_windows_machine(target) -> bool:
    # if port 135 and 445 open
    if is_port_open(target, 135) and is_port_open(target, 445):
        return True
    else:
        return False


def is_target_a_windows_domain_controller(target) -> bool:
    # if port 135 and 445 and 88 open
    if is_target_a_windows_machine(target) and is_port_open(target, 88):
        return True
    else:
        return False


def is_port_open(target, port) -> bool:
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        s.settimeout(1)
        return s.connect_ex((target, port)) == 0


def is_http_accessible(target, port, config, scheme="http"):
    url = "%s://%s:%d/" % (scheme, target, port)
    try:
        r = requests.get(
            url,
            timeout=config.request_timeout,
            proxies=config.request_proxies,
            verify=(not (config.request_no_check_certificate))
        )
        return True
    except Exception as e:
        config.debug("Error in is_http_accessible('%s', %d, '%s'): %s " % (target, port, scheme, e))
        return False


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : update_db.py
# Author             : Podalirius (@podalirius_)
# Date created       : 5 Dec 2022


import json
import os
import requests
from bs4 import BeautifulSoup
import glob


def parse_vulns(vulnerabilities_in_this_ver, Version, Language, Update, Edition, CVES):
    r = requests.get("https://www.cvedetails.com/%s" % vulnerabilities_in_this_ver)
    soup = BeautifulSoup(r.content, 'lxml')

    table = soup.find('table', attrs={"id": "vulnslisttable"})
    for tr in table.findAll("tr"):
        tds = tr.findAll('td')
        if len(tds) == 15:
            id = tds[0].text.strip()

            cve_id = tds[1].text.strip()
            print("   [>] Parsing %s" % cve_id)
            cve_id_link = "https://www.cvedetails.com" + tds[1].find('a')['href']

            cwe_id = tds[2].text.strip()
            cwe_id_link = None
            if tds[2].find('a') is not None:
                cwe_id_link = "https://www.cvedetails.com" + tds[2].find('a')['href']

            if cve_id not in CVES.keys():
                CVES[cve_id] = {}

            if "cve" not in CVES[cve_id].keys():
                CVES[cve_id]["cve"] = {}
            number_of_exploits = tds[3].text.strip()
            CVES[cve_id]["cve"]["id"] = cve_id
            CVES[cve_id]["cve"]["year"] = int(cve_id.split('-')[1])
            CVES[cve_id]["cve"]["vuln_type"] = tds[4].text.strip()
            CVES[cve_id]["cve"]["publish_date"] = tds[5].text.strip()
            CVES[cve_id]["cve"]["update_date"] = tds[6].text.strip()

            if "cvss" not in CVES[cve_id].keys():
                CVES[cve_id]["cvss"] = {}
            CVES[cve_id]["cvss"]["score"] = tds[7].text.strip()

            risk_levels = ["None", "Low", "Medium", "High", "Critical"]
            criticity = "None"
            if float(CVES[cve_id]["cvss"]["score"]) == 0:
                criticity = risk_levels[0]
            elif 0 < float(CVES[cve_id]["cvss"]["score"]) < 4:
                criticity = risk_levels[1]
            elif 4 <= float(CVES[cve_id]["cvss"]["score"]) < 7:
                criticity = risk_levels[2]
            elif 7 <= float(CVES[cve_id]["cvss"]["score"]) < 9:
                criticity = risk_levels[3]
            elif 9 <= float(CVES[cve_id]["cvss"]["score"]) < 10:
                criticity = risk_levels[4]

            CVES[cve_id]["cvss"]["criticity"] = criticity
            CVES[cve_id]["cvss"]["gained_access_level"] = tds[8].text.strip()
            CVES[cve_id]["cvss"]["access"] = tds[9].text.strip()
            CVES[cve_id]["cvss"]["complexity"] = tds[10].text.strip()
            CVES[cve_id]["cvss"]["confidentiality"] = tds[8].text.strip()
            CVES[cve_id]["cvss"]["integrity"] = tds[8].text.strip()
            CVES[cve_id]["cvss"]["availablility"] = tds[8].text.strip()

            if "affected_versions" not in CVES[cve_id].keys():
                CVES[cve_id]["affected_versions"] = []
            CVES[cve_id]["affected_versions"].append({
                "tag": (Version+'-'+Update if Update != '*' else Version),
                "version": Version,
                "language": Language,
                "update": Update,
                "edition": Edition
            })

            if "references" not in CVES[cve_id].keys():
                CVES[cve_id]["references"] = []
            CVES[cve_id]["references"].append("https://nvd.nist.gov/vuln/detail/%s" % cve_id)

            r = requests.get(cve_id_link)
            soup = BeautifulSoup(r.content, 'lxml')

            cvedetailssummary = soup.find('div', attrs={"class": "cvedetailssummary"})
            CVES[cve_id]["description"] = cvedetailssummary.text.strip().split('Publish Date : ')[0].strip()

            vulnrefstable = soup.find('table', attrs={"id": "vulnrefstable"})
            for tr in vulnrefstable.findAll('tr'):
                links = [a['href'] for a in tr.findAll('a')]
                for link in links:
                    CVES[cve_id]["references"].append(link)
            CVES[cve_id]["references"] = list(sorted(set(CVES[cve_id]["references"])))


if __name__ == '__main__':
    CVES = {}

    if os.path.exists("./vulnerabilities/"):
        for file in glob.glob("./vulnerabilities/*/*.json"):
            f = open(file, "r")
            data = json.loads(f.read())
            f.close()
            CVES[data["cve"]["id"]] = data

    r = requests.get("https://www.cvedetails.com/version-list/45/887/2/Apache-Tomcat.html?sha=1e26d2dc4f7319bbf6b0bf066415a3daf97151c8&order=1&trc=986")
    soup = BeautifulSoup(r.content, 'lxml')

    pagingb = soup.find('div', attrs={"id": "pagingb", "class": "paging"})
    pages = list(map(int, [a.text.strip() for a in pagingb.findAll('a')]))

    for page_number in pages:
        r = requests.get("https://www.cvedetails.com/version-list/45/887/%d/Apache-Tomcat.html?sha=1e26d2dc4f7319bbf6b0bf066415a3daf97151c8&order=1&trc=986" % page_number)
        soup = BeautifulSoup(r.content, 'lxml')

        table = soup.find('table', attrs={"class": "listtable"})
        print("[>] Parsing page %d/%d" % (page_number, pages[-1]))
        for tr in table.findAll("tr"):
            tds = tr.findAll('td')
            if len(tds) == 6:
                Version = tds[0].text.strip()
                Language = tds[1].text.strip()
                Update = tds[2].text.strip()
                Edition = tds[3].text.strip()
                Number_of_Vulnerabilities = int(tds[4].text.strip())
                links = tds[5].findAll('a')
                version_details = links[0]["href"]
                vulnerabilities_in_this_ver = links[1]["href"]
                if not (Version == "*" and Language == "*" and Update == "*" and Edition == "*"):
                    parse_vulns(vulnerabilities_in_this_ver, Version, Language, Update, Edition, CVES)

    for cve_id, cve_data in CVES.items():
        save_path = "./vulnerabilities/%d/%s.json" % (cve_data["cve"]["year"], cve_id)
        if not os.path.exists(os.path.dirname(save_path)):
            os.makedirs(os.path.dirname(save_path))

        if not os.path.exists(save_path):
            f = open(save_path, 'w')
            f.write(json.dumps(cve_data, indent=4))
            f.close()
        else:
            print("[+] Skipping CVE-%s-%s because it already exists." % (cve_data["cve"]["year"], cve_id))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : Reporter.py
# Author             : Podalirius (@podalirius_)
# Date created       : 31 Jul 2022

import json
import os.path
import sqlite3
import xlsxwriter


class Reporter(object):
    """
    Documentation for class Reporter
    """

    data = {}

    def __init__(self, config, vulns_db):
        super(Reporter, self).__init__()
        self.config = config
        self.vulns_db = vulns_db
        self._new_results = []

    def report_result(self, computer_ip, computer_port, tomcat_version, manager_accessible, credentials_found):
        computer_port = str(computer_port)

        finding = {}
        finding["computer_ip"] = computer_ip
        finding["computer_port"] = computer_port
        finding["tomcat_version"] = tomcat_version
        finding["manager_accessible"] = manager_accessible
        finding["credentials_found"] = credentials_found

        if computer_ip not in self.data.keys():
            self.data[computer_ip] = {}
        if str(computer_port) not in self.data[computer_ip].keys():
            self.data[computer_ip][computer_port] = {}
        self.data[computer_ip][computer_port] = finding
        self._new_results.append(finding)

    def print_new_results(self):
        try:
            for finding in self._new_results:

                # List of cves
                cve_str = ""
                if self.config.list_cves_mode == True:
                    cve_list = self.vulns_db.get_vulnerabilities_of_version_sorted_by_criticity(finding["tomcat_version"], colors=True, reverse=True)
                    if len(cve_list) != 0:
                        cve_str = "CVEs: %s" % ', '.join(cve_list)

                # credentials_str = "username:%s\npassword:%s" % (credentials_found[0][1]["username"], credentials_found[0][1]["password"])

                if finding["manager_accessible"]:
                    print("[>] [Apache Tomcat/\x1b[1;95m%s\x1b[0m] on \x1b[1;93m%s\x1b[0m:\x1b[1;93m%s\x1b[0m (manager:\x1b[1;92maccessible\x1b[0m) %s\x1b[0m " % (
                            finding["tomcat_version"],
                            finding["computer_ip"],
                            finding["computer_port"],
                            cve_str
                        )
                    )
                    if len(finding["credentials_found"]) != 0:
                        for statuscode, creds in finding["credentials_found"]:
                            if len(creds["description"]) != 0:
                                print("  | Valid user: \x1b[1;92m%s\x1b[0m | password:\x1b[1;92m%s\x1b[0m | \x1b[94m%s\x1b[0m" % (creds["username"], creds["password"], creds["description"]))
                            else:
                                print("  | Valid user: \x1b[1;92m%s\x1b[0m | password:\x1b[1;92m%s\x1b[0m" % (creds["username"], creds["password"]))

                else:
                    print("[>] [Apache Tomcat/\x1b[1;95m%s\x1b[0m] on \x1b[1;93m%s\x1b[0m:\x1b[1;93m%s\x1b[0m (manager:\x1b[1;91mnot accessible\x1b[0m) %s\x1b[0m " % (
                            finding["tomcat_version"],
                            finding["computer_ip"],
                            finding["computer_port"],
                            cve_str
                        )
                    )

                self._new_results.remove(finding)
        except Exception as e:
            if self.config.debug_mode:
                print("[Error in %s] %s" % (__name__, e))

    def export_xlsx(self, path_to_file):
        basepath = os.path.dirname(path_to_file)
        filename = os.path.basename(path_to_file)
        if basepath not in [".", ""]:
            if not os.path.exists(basepath):
                os.makedirs(basepath)
            path_to_file = basepath + os.path.sep + filename
        else:
            path_to_file = filename

        workbook = xlsxwriter.Workbook(path_to_file)
        worksheet = workbook.add_worksheet()

        header_format = workbook.add_format({'bold': 1})
        header_fields = ["Computer IP", "Port", "Apache tomcat version", "Manager accessible", "Default credentials found", "CVEs on this version"]
        for k in range(len(header_fields)):
            worksheet.set_column(k, k + 1, len(header_fields[k]) + 3)
        worksheet.set_row(0, 20, header_format)
        worksheet.write_row(0, 0, header_fields)

        row_id = 1
        for computername in self.data.keys():
            computer = self.data[computername]
            for port in computer.keys():
                cve_list = self.vulns_db.get_vulnerabilities_of_version_sorted_by_criticity(computer[port]["tomcat_version"], colors=False, reverse=True)
                cve_str = ', '.join([cve["cve"]["id"] for cve in cve_list])

                data = [
                    computer[port]["computer_ip"],
                    computer[port]["computer_port"],
                    computer[port]["tomcat_version"],
                    str(computer[port]["manager_accessible"]).upper(),
                    computer[port]["default_credentials"],
                    cve_str
                ]
                worksheet.write_row(row_id, 0, data)
                row_id += 1
        worksheet.autofilter(0, 0, row_id, len(header_fields) - 1)
        workbook.close()

    def export_json(self, path_to_file):
        basepath = os.path.dirname(path_to_file)
        filename = os.path.basename(path_to_file)
        if basepath not in [".", ""]:
            if not os.path.exists(basepath):
                os.makedirs(basepath)
            path_to_file = basepath + os.path.sep + filename
        else:
            path_to_file = filename
        f = open(path_to_file, 'w')
        f.write(json.dumps(self.data, indent=4))
        f.close()

    def export_sqlite(self, path_to_file):
        basepath = os.path.dirname(path_to_file)
        filename = os.path.basename(path_to_file)
        if basepath not in [".", ""]:
            if not os.path.exists(basepath):
                os.makedirs(basepath)
            path_to_file = basepath + os.path.sep + filename
        else:
            path_to_file = filename

        conn = sqlite3.connect(path_to_file)
        cursor = conn.cursor()
        cursor.execute("CREATE TABLE IF NOT EXISTS results(computer_ip VARCHAR(255), computer_port INTEGER, tomcat_version VARCHAR(255), manager_accessible VARCHAR(255), default_credentials VARCHAR(255), cves INTEGER);")
        for computername in self.data.keys():
            computer = self.data[computername]
            for port in computer.keys():
                cve_list = self.vulns_db.get_vulnerabilities_of_version_sorted_by_criticity(computer[port]["tomcat_version"], colors=False, reverse=True)
                cve_str = ', '.join([cve["cve"]["id"] for cve in cve_list])

                cursor.execute("INSERT INTO results VALUES (?, ?, ?, ?, ?, ?)", (
                        computer[port]["computer_ip"],
                        computer[port]["computer_port"],
                        computer[port]["tomcat_version"],
                        str(computer[port]["manager_accessible"]).upper(),
                        computer[port]["default_credentials"],
                        cve_str
                    )
                )
        conn.commit()
        conn.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : Config.py
# Author             : Podalirius (@podalirius_)
# Date created       : 31 Jul 2022

import json
import os


class Config(object):
    """
    Documentation for class Config
    """

    request_timeout = 1
    request_proxies = {}
    request_no_check_certificate = True

    request_available_schemes = ["http"]

    list_cves_mode = False

    debug_mode = False
    verbose_mode = False

    credentials = {}

    def __init__(self):
        super(Config, self).__init__()
        self.__load_default_credentials()

    def debug(self, msg):
        if self.debug_mode:
            print("[debug]", msg)

    def __load_default_credentials(self):
        self.credentials = {}
        path_to_creds = os.path.dirname(__file__) + os.path.sep + 'data' + os.path.sep + 'credentials.json'
        f = open(path_to_creds, 'r')
        self.credentials = json.loads(f.read())["credentials"]
        f.close()
        return None

    def load_credentials_from_options(self, username, password, usernames_file, passwords_file):
        usernames = []
        passwords = []

        if username is not None:
            usernames.append(username)
        if usernames_file is not None:
            f = open(usernames_file, "r")
            for line in f.readlines():
                usernames.append(line.strip())
            f.close()

        if password is not None:
            passwords.append(password)
        if passwords_file is not None:
            f = open(passwords_file, "r")
            for line in f.readlines():
                passwords.append(line.strip())
            f.close()

        if len(usernames) != 0 and len(passwords) != 0:
            self.credentials = {"credentials": []}
            for username in usernames:
                for password in passwords:
                    self.credentials["credentials"].append({
                        "username": username,
                        "password": password,
                        "description": ""
                    })
            return True
        else:
            return False

    # Get / Set functions

    def get_request_available_schemes(self):
        return self.request_available_schemes

    def set_request_available_schemes(self, only_http, only_https):
        self.request_available_schemes = []
        if only_https:
            self.request_available_schemes.append("https")
        elif only_http:
            self.request_available_schemes.append("http")
        else:
            self.request_available_schemes.append("http")
            self.request_available_schemes.append("https")

    def get_request_timeout(self):
        return self.request_timeout

    def set_request_timeout(self, value):
        self.request_timeout = value

    def get_request_no_check_certificate(self):
        return self.request_no_check_certificate

    def set_request_no_check_certificate(self, value):
        self.request_no_check_certificate = value

    def get_list_cves_mode(self):
        return self.list_cves_mode

    def set_list_cves_mode(self, value):
        self.list_cves_mode = value

    def get_debug_mode(self):
        return self.debug_mode

    def set_debug_mode(self, value):
        if value == True:
            self.verbose_mode = True
        self.debug_mode = value

    def get_verbose_mode(self):
        return self.verbose_mode

    def set_verbose_mode(self, value):
        self.verbose_mode = value

    def get_request_proxies(self):
        return self.request_proxies

    def set_request_proxies(self, proxy_ip, proxy_port, protocol=None):
        if proxy_ip is not None and proxy_port is not None:
            if protocol is None:
                self.request_proxies = {
                    "http": "http://%s:%d/" % (proxy_ip, proxy_port),
                    "https": "https://%s:%d/" % (proxy_ip, proxy_port)
                }
            else:
                self.request_proxies[protocol] = "%s://%s:%d/" % (protocol, proxy_ip, proxy_port)
        return self.request_proxies

    def clear_request_proxies(self):
        self.request_proxies = {}
